@@ -124,10 +124,12 @@ void gegl_buffer_header_init (GeglBufferHeader *header,
 
 void gegl_tile_entry_destroy (GeglBufferTile *entry);
 
-GeglBufferItem *gegl_buffer_read_header(int i,
-                                        goffset      *offset);
-GList          *gegl_buffer_read_index (int i,
-                                        goffset      *offset);
+GeglBufferItem *gegl_buffer_read_header(int      i,
+                                        goffset *offset,
+                                        gchar   *map);
+GList          *gegl_buffer_read_index (int      i,
+                                        goffset *offset,
+                                        gchar   *map);
 
 #define struct_check_padding(type, size) \
   if (sizeof (type) != size) \
