@@ -18,6 +18,8 @@ gegl_buffer_linear_new2 (const GeglRectangle *extent,
                          const Babl          *format,
                          gint                 rowstride)
 {
+  GeglBuffer *buffer;
+
   if (extent==NULL)
     {
       g_error ("got a NULL extent");
@@ -33,17 +35,21 @@ gegl_buffer_linear_new2 (const GeglRectangle *extent,
    * requesting the correct parameters when creating the
    * buffer
    */
-  return g_object_new (GEGL_TYPE_BUFFER,
+  buffer = g_object_new (GEGL_TYPE_BUFFER,
                        "x",          extent->x,
                        "y",          extent->y,
-                       "shift-x",    extent->x,
-                       "shift-y",    extent->y,
+                       "shift-x",    0,
+                       "shift-y",    0,
                        "width",      extent->width,
                        "height",     extent->height,
                        "tile-width", rowstride,
                        "tile-height", extent->height,
                        "format", format,
                        NULL);
+
+  g_object_set_data (G_OBJECT (buffer), "is-linear", (void*)0xf00);
+
+  return buffer;
 }
 
 GeglBuffer *
@@ -66,7 +72,7 @@ gegl_buffer_linear_new_from_data (const gpointer       data,
                                   const Babl          *format,
                                   const GeglRectangle *extent,
                                   gint                 rowstride,
-                                  GCallback            destroy_fn,
+                                  GDestroyNotify       destroy_fn,
                                   gpointer             destroy_fn_data)
 {
   GeglBuffer *buffer;
@@ -93,7 +99,7 @@ gegl_buffer_linear_new_from_data (const gpointer       data,
     gegl_tile_set_data_full (tile,
                              (gpointer) data,
                              babl_format_get_bytes_per_pixel (format) * rowstride * extent->height,
-                             (GeglDestroyNotify) destroy_fn,
+                             destroy_fn,
                              destroy_fn_data);
 
     if (buffer->tile_storage->cache)
@@ -125,18 +131,18 @@ gegl_buffer_linear_open (GeglBuffer          *buffer,
                          const Babl          *format)   /* if NULL, from buf */
 {
   if (!format)
-    format = buffer->format;
+    format = buffer->soft_format;
 
   if (extent == NULL)
     extent=&buffer->extent;
 
   /*gegl_buffer_lock (buffer);*/
-  g_mutex_lock (buffer->tile_storage->mutex);
+  g_mutex_lock (&buffer->tile_storage->mutex);
   if (extent->x     == buffer->extent.x &&
       extent->y     == buffer->extent.y &&
       extent->width == buffer->tile_width &&
       extent->height <= buffer->tile_height &&
-      buffer->format == format)
+      buffer->soft_format == format)
     {
       GeglTile *tile;
 
@@ -199,7 +205,7 @@ gegl_buffer_linear_open (GeglBuffer          *buffer,
     if(rowstride)*rowstride = rs;
 
     info->buf = gegl_malloc (rs * info->extent.height);
-    gegl_buffer_get_unlocked (buffer, 1.0, &info->extent, format, info->buf, rs);
+    gegl_buffer_get_unlocked (buffer, 1.0, &info->extent, format, info->buf, rs, GEGL_ABYSS_NONE);
     return info->buf;
   }
   return NULL;
@@ -242,19 +248,19 @@ gegl_buffer_linear_close (GeglBuffer *buffer,
               linear_buffers = g_list_remove (linear_buffers, info);
               g_object_set_data (G_OBJECT (buffer), "linear-buffers", linear_buffers);
 
-              g_mutex_unlock (buffer->tile_storage->mutex);
+              g_mutex_unlock (&buffer->tile_storage->mutex);
               /* XXX: potential race */
-              gegl_buffer_set (buffer, &info->extent, info->format, info->buf, 0);
+              gegl_buffer_set (buffer, &info->extent, 0, info->format, info->buf, 0);
 
               gegl_free (info->buf);
               g_free (info);
 
-              g_mutex_lock (buffer->tile_storage->mutex);
+              g_mutex_lock (&buffer->tile_storage->mutex);
               break;
             }
         }
     }
   /*gegl_buffer_unlock (buffer);*/
-  g_mutex_unlock (buffer->tile_storage->mutex);
+  g_mutex_unlock (&buffer->tile_storage->mutex);
   return;
 }
