@@ -28,8 +28,11 @@
 #include "gegl-buffer-private.h"
 #include "gegl-tile.h"
 #include "gegl-tile-handler-cache.h"
+#include "gegl-tile-storage.h"
 #include "gegl-debug.h"
 
+#include "gegl-buffer-cl-cache.h"
+
 /*
 #define GEGL_DEBUG_CACHE_HITS
 */
@@ -38,17 +41,16 @@ typedef struct CacheItem
 {
   GeglTileHandlerCache *handler; /* The specific handler that cached this item*/
   GeglTile *tile;                /* The tile */
+  GList     link;                /*  Link in the cache_queue, to avoid
+                                  *  queue lookups involving g_list_find() */
 
   gint      x;                   /* The coordinates this tile was cached for */
   gint      y;
   gint      z;
 } CacheItem;
 
-struct _GeglTileHandlerCache
-{
-  GeglTileHandler parent_instance;
-  GSList *free_list;
-};
+#define LINK_GET_ITEM(link) \
+        ((CacheItem *) ((guchar *) link - G_STRUCT_OFFSET (CacheItem, link)))
 
 
 static void       gegl_tile_handler_cache_dispose    (GObject              *object);
@@ -82,11 +84,11 @@ static void       gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cach
                                                       gint                  z);
 
 
-static GStaticMutex mutex                 = G_STATIC_MUTEX_INIT;
+static GMutex       mutex                 = { 0, };
 static GQueue      *cache_queue           = NULL;
 static GHashTable  *cache_ht              = NULL;
 static gint         cache_wash_percentage = 20;
-static gint         cache_total           = 0; /* approximate amount of bytes stored */
+static guint64      cache_total           = 0; /* approximate amount of bytes stored */
 #ifdef GEGL_DEBUG_CACHE_HITS
 static gint         cache_hits            = 0;
 static gint         cache_misses          = 0;
@@ -111,6 +113,59 @@ gegl_tile_handler_cache_init (GeglTileHandlerCache *cache)
   gegl_tile_cache_init ();
 }
 
+
+static void
+gegl_tile_handler_cache_reinit_buffer_tiles (gpointer itm,
+                                             gpointer userdata)
+{
+  CacheItem *item;
+  item = itm;
+  if (item->handler == userdata)
+    {
+      GeglTileHandlerCache *cache = userdata;
+      cache->free_list = g_slist_prepend (cache->free_list, item);
+    }
+}
+
+static void
+gegl_tile_handler_cache_reinit (GeglTileHandlerCache *cache)
+{
+  CacheItem            *item;
+  GSList               *iter;
+
+  if (cache->tile_storage->hot_tile)
+    {
+      gegl_tile_unref (cache->tile_storage->hot_tile);
+      cache->tile_storage->hot_tile = NULL;
+    }
+
+  if (!cache->count)
+    return;
+
+  g_mutex_lock (&mutex);
+  /* only throw out items belonging to this cache instance */
+
+  cache->free_list = NULL;
+  g_queue_foreach (cache_queue, gegl_tile_handler_cache_reinit_buffer_tiles, cache);
+  for (iter = cache->free_list; iter; iter = g_slist_next (iter))
+    {
+      item = iter->data;
+      if (item->tile)
+        {
+          cache_total -= item->tile->size;
+          gegl_tile_mark_as_stored (item->tile); /* to avoid saving */
+          gegl_tile_unref (item->tile);
+          cache->count--;
+        }
+      g_queue_unlink (cache_queue, &item->link);
+      g_hash_table_remove (cache_ht, item);
+      g_slice_free (CacheItem, item);
+    }
+  g_slist_free (cache->free_list);
+  cache->free_list = NULL;
+  g_mutex_unlock (&mutex);
+}
+
 static void
 gegl_tile_handler_cache_dispose_buffer_tiles (gpointer itm,
                                               gpointer userdata)
@@ -131,8 +186,7 @@ gegl_tile_handler_cache_dispose (GObject *object)
   CacheItem            *item;
   GSList               *iter;
 
-  g_static_mutex_lock (&mutex);
-  cache = GEGL_TILE_HANDLER_CACHE (object);
+  cache = (GeglTileHandlerCache*) (object);
 
   /* only throw out items belonging to this cache instance */
 
@@ -141,23 +195,33 @@ gegl_tile_handler_cache_dispose (GObject *object)
    * buffer destructions, to avoid the overhead of walking the full queue for
    * every tiny buffer being destroyed.
    */
-  g_queue_foreach (cache_queue, gegl_tile_handler_cache_dispose_buffer_tiles, cache);
-  for (iter = cache->free_list; iter; iter = g_slist_next (iter))
-    {
-        item = iter->data;
-        if (item->tile)
-          {
-            cache_total -= item->tile->size;
-            gegl_tile_unref (item->tile);
-          }
-        g_queue_remove (cache_queue, item);
-        g_hash_table_remove (cache_ht, item);
-        g_slice_free (CacheItem, item);
-    }
-  g_slist_free (cache->free_list);
-  cache->free_list = NULL;
-  g_static_mutex_unlock (&mutex);
 
+  if (cache->count)
+    {
+      g_mutex_lock (&mutex);
+      g_queue_foreach (cache_queue, gegl_tile_handler_cache_dispose_buffer_tiles, cache);
+      for (iter = cache->free_list; iter; iter = g_slist_next (iter))
+        {
+            item = iter->data;
+            if (item->tile)
+              {
+                cache_total -= item->tile->size;
+                gegl_tile_unref (item->tile);
+                cache->count--;
+              }
+            g_queue_unlink (cache_queue, &item->link);
+            g_hash_table_remove (cache_ht, item);
+            g_slice_free (CacheItem, item);
+        }
+      g_slist_free (cache->free_list);
+      cache->free_list = NULL;
+      g_mutex_unlock (&mutex);
+    }
+
+  if (cache->count < 0)
+    {
+      g_warning ("cache-handler tile balance not zero: %i\n", cache->count);
+    }
   G_OBJECT_CLASS (gegl_tile_handler_cache_parent_class)->dispose (object);
 }
 
@@ -167,10 +231,13 @@ gegl_tile_handler_cache_get_tile_command (GeglTileSource *tile_store,
                                           gint        y,
                                           gint        z)
 {
-  GeglTileHandlerCache *cache    = GEGL_TILE_HANDLER_CACHE (tile_store);
-  GeglTileSource       *source = GEGL_TILE_HANDLER (tile_store)->source;
+  GeglTileHandlerCache *cache    = (GeglTileHandlerCache*) (tile_store);
+  GeglTileSource       *source   = ((GeglTileHandler*) (tile_store))->source;
   GeglTile             *tile     = NULL;
 
+  if (G_UNLIKELY (gegl_cl_is_accelerated ()))
+    gegl_buffer_cl_cache_flush2 (cache, NULL);
+
   tile = gegl_tile_handler_cache_get_tile (cache, x, y, z);
   if (tile)
     {
@@ -200,8 +267,8 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
                                  gint             z,
                                  gpointer         data)
 {
-  GeglTileHandler      *handler = GEGL_TILE_HANDLER (tile_store);
-  GeglTileHandlerCache *cache   = GEGL_TILE_HANDLER_CACHE (handler);
+  GeglTileHandler      *handler = (GeglTileHandler*) (tile_store);
+  GeglTileHandlerCache *cache   = (GeglTileHandlerCache*) (handler);
 
   switch (command)
     {
@@ -209,15 +276,21 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
         {
           GList     *link;
 
-          for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
-            {
-              CacheItem *item = link->data;
-              GeglTile  *tile = item->tile;
+          if (gegl_cl_is_accelerated ())
+            gegl_buffer_cl_cache_flush2 (cache, NULL);
 
-              if (tile != NULL &&
-                  item->handler == cache)
+          if (cache->count)
+            {
+              for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
                 {
-                  gegl_tile_store (tile);
+                  CacheItem *item = LINK_GET_ITEM (link);
+                  GeglTile  *tile = item->tile;
+
+                  if (tile != NULL &&
+                      item->handler == cache)
+                    {
+                      gegl_tile_store (tile);
+                    }
                 }
             }
         }
@@ -251,6 +324,9 @@ gegl_tile_handler_cache_command (GeglTileSource  *tile_store,
       case GEGL_TILE_VOID:
         gegl_tile_handler_cache_void (cache, x, y, z);
         break;
+      case GEGL_TILE_REINIT:
+        gegl_tile_handler_cache_reinit (cache);
+        break;
       default:
         break;
     }
@@ -274,7 +350,7 @@ gegl_tile_handler_cache_wash (GeglTileHandlerCache *cache)
 
   for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
     {
-      CacheItem *item = link->data;
+      CacheItem *item = LINK_GET_ITEM (link);
       GeglTile  *tile = item->tile;
 
       count++;
@@ -291,6 +367,22 @@ gegl_tile_handler_cache_wash (GeglTileHandlerCache *cache)
   return FALSE;
 }
 
+static inline CacheItem *
+cache_lookup (GeglTileHandlerCache *cache,
+              gint                  x,
+              gint                  y,
+              gint                  z)
+{
+  CacheItem key;
+
+  key.x       = x;
+  key.y       = y;
+  key.z       = z;
+  key.handler = cache;
+
+  return g_hash_table_lookup (cache_ht, &key);
+}
+
 /* returns the requested Tile if it is in the cache, NULL otherwize.
  */
 static GeglTile *
@@ -300,23 +392,20 @@ gegl_tile_handler_cache_get_tile (GeglTileHandlerCache *cache,
                                   gint                  z)
 {
   CacheItem *result;
-  CacheItem pin;
 
-  pin.x = x;
-  pin.y = y;
-  pin.z = z;
-  pin.handler = cache;
+  if (cache->count == 0)
+    return NULL;
 
-  g_static_mutex_lock (&mutex);
-  result = g_hash_table_lookup (cache_ht, &pin);
+  g_mutex_lock (&mutex);
+  result = cache_lookup (cache, x, y, z);
   if (result)
     {
-      g_queue_remove (cache_queue, result);
-      g_queue_push_head (cache_queue, result);
-      g_static_mutex_unlock (&mutex);
+      g_queue_unlink (cache_queue, &result->link);
+      g_queue_push_head_link (cache_queue, &result->link);
+      g_mutex_unlock (&mutex);
       return gegl_tile_ref (result->tile);
     }
-  g_static_mutex_unlock (&mutex);
+  g_mutex_unlock (&mutex);
   return NULL;
 }
 
@@ -340,12 +429,14 @@ gegl_tile_handler_cache_has_tile (GeglTileHandlerCache *cache,
 static gboolean
 gegl_tile_handler_cache_trim (GeglTileHandlerCache *cache)
 {
-  CacheItem *last_writable;
+  GList *link;
 
-  last_writable = g_queue_pop_tail (cache_queue);
+  link = g_queue_pop_tail_link (cache_queue);
 
-  if (last_writable != NULL)
+  if (link != NULL)
     {
+      CacheItem *last_writable = LINK_GET_ITEM (link);
+
       g_hash_table_remove (cache_ht, last_writable);
       cache_total  -= last_writable->tile->size;
       gegl_tile_unref (last_writable->tile);
@@ -362,32 +453,21 @@ gegl_tile_handler_cache_invalidate (GeglTileHandlerCache *cache,
                                     gint                  y,
                                     gint                  z)
 {
-  GList *link;
+  CacheItem *item;
 
-  g_static_mutex_lock (&mutex);
-  for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
+  g_mutex_lock (&mutex);
+  item = cache_lookup (cache, x, y, z);
+  if (item)
     {
-      CacheItem *item = link->data;
-      GeglTile  *tile = item->tile;
-
-      if (tile != NULL &&
-          item->x == x &&
-          item->y == y &&
-          item->z == z &&
-          item->handler == cache)
-        {
-          cache_total  -= item->tile->size;
-          tile->tile_storage = NULL;
-          gegl_tile_mark_as_stored (tile); /* to cheat it out of being stored */
-          gegl_tile_unref (tile);
-          g_hash_table_remove (cache_ht, item);
-          g_slice_free (CacheItem, item);
-          g_queue_delete_link (cache_queue, link);
-          g_static_mutex_unlock (&mutex);
-          return;
-        }
+      cache_total -= item->tile->size;
+      item->tile->tile_storage = NULL;
+      gegl_tile_mark_as_stored (item->tile); /* to cheat it out of being stored */
+      gegl_tile_unref (item->tile);
+      g_queue_unlink (cache_queue, &item->link);
+      g_hash_table_remove (cache_ht, item);
+      g_slice_free (CacheItem, item);
     }
-  g_static_mutex_unlock (&mutex);
+  g_mutex_unlock (&mutex);
 }
 
 
@@ -397,34 +477,21 @@ gegl_tile_handler_cache_void (GeglTileHandlerCache *cache,
                               gint                  y,
                               gint                  z)
 {
-  GList *link;
+  CacheItem *item;
 
-  if (!cache_queue)
-    return;
-
-  g_static_mutex_lock (&mutex);
-  for (link = g_queue_peek_head_link (cache_queue); link; link = link->next)
+  g_mutex_lock (&mutex);
+  item = cache_lookup (cache, x, y, z);
+  if (item)
     {
-      CacheItem *item = link->data;
-      GeglTile  *tile = item->tile;
-
-      if (tile != NULL &&
-          item->x == x &&
-          item->y == y &&
-          item->z == z &&
-          item->handler == cache)
-        {
-          gegl_tile_void (tile);
-          cache_total -= item->tile->size;
-          gegl_tile_unref (tile);
-          g_hash_table_remove (cache_ht, item);
-          g_slice_free (CacheItem, item);
-          g_queue_delete_link (cache_queue, link);
-          g_static_mutex_unlock (&mutex);
-          return;
-        }
+      gegl_tile_void (item->tile);
+      cache_total -= item->tile->size;
+      gegl_tile_unref (item->tile);
+      g_queue_unlink (cache_queue, &item->link);
+      g_hash_table_remove (cache_ht, item);
+      g_slice_free (CacheItem, item);
+      cache->count --;
     }
-  g_static_mutex_unlock (&mutex);
+  g_mutex_unlock (&mutex);
 }
 
 void
@@ -436,30 +503,39 @@ gegl_tile_handler_cache_insert (GeglTileHandlerCache *cache,
 {
   CacheItem *item = g_slice_new (CacheItem);
 
-  item->handler = cache;
-  item->tile    = gegl_tile_ref (tile);
-  item->x       = x;
-  item->y       = y;
-  item->z       = z;
+  item->handler   = cache;
+  item->tile      = gegl_tile_ref (tile);
+  item->link.data = item;
+  item->link.next = NULL;
+  item->link.prev = NULL;
+  item->x         = x;
+  item->y         = y;
+  item->z         = z;
 
-  g_static_mutex_lock (&mutex);
+  // XXX : remove entry if it already exists
+  gegl_tile_handler_cache_void (cache, x, y, z);
+
+  g_mutex_lock (&mutex);
   cache_total  += item->tile->size;
-  g_queue_push_head (cache_queue, item);
+  g_queue_push_head_link (cache_queue, &item->link);
+
+  cache->count ++;
+
 
   g_hash_table_insert (cache_ht, item, item);
 
-  while (cache_total > gegl_config()->cache_size)
+  while (cache_total > gegl_config()->tile_cache_size)
     {
 #ifdef GEGL_DEBUG_CACHE_HITS
-      GEGL_NOTE(GEGL_DEBUG_CACHE, "cache_total:%i > cache_size:%i", cache_total, gegl_config()->cache_size);
+      GEGL_NOTE(GEGL_DEBUG_CACHE, "cache_total:"G_GUINT64_FORMAT" > cache_size:"G_GUINT64_FORMAT, cache_total, gegl_config()->tile_cache_size);
       GEGL_NOTE(GEGL_DEBUG_CACHE, "%f%% hit:%i miss:%i  %i]", cache_hits*100.0/(cache_hits+cache_misses), cache_hits, cache_misses, g_queue_get_length (cache_queue));
 #endif
       gegl_tile_handler_cache_trim (cache);
     }
-  g_static_mutex_unlock (&mutex);
+  g_mutex_unlock (&mutex);
 }
 
-GeglTileHandlerCache *
+GeglTileHandler *
 gegl_tile_handler_cache_new (void)
 {
   return g_object_new (GEGL_TYPE_TILE_HANDLER_CACHE, NULL);
@@ -520,7 +596,10 @@ void
 gegl_tile_cache_destroy (void)
 {
   if (cache_queue)
-    g_queue_free (cache_queue);
+    {
+      while (g_queue_pop_head_link (cache_queue));
+      g_queue_free (cache_queue);
+    }
   if (cache_ht)
     g_hash_table_destroy (cache_ht);
   cache_queue = NULL;
