@@ -48,8 +48,8 @@ static GType    type_name##_type_id = 0;                                \
 static void     type_name##_class_chant_intern_init (gpointer klass)    \
   {                                                                     \
     gegl_chant_parent_class = g_type_class_peek_parent (klass);         \
-    type_name##_class_init ((TypeName##Class*) klass);                  \
     gegl_chant_class_intern_init (klass);                               \
+    type_name##_class_init ((TypeName##Class*) klass);                  \
   }                                                                     \
 GType                                                                   \
 type_name##_get_type (void)                                             \
@@ -334,29 +334,56 @@ gegl_module_register (GTypeModule *module)
 }
 #endif
 
+/* enum registration */
+#define gegl_chant_int(name, nick, min, max, def, blurb)
+#define gegl_chant_int_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb)
+#define gegl_chant_double(name, nick, min, max, def, blurb)
+#define gegl_chant_double_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb)
+#define gegl_chant_boolean(name, nick, def, blurb)
+#define gegl_chant_string(name, nick, def, blurb)
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)
+#define gegl_chant_file_path(name, nick, def, blurb)
+#define gegl_chant_multiline(name, nick, def, blurb)
+#define gegl_chant_object(name,nick,  blurb)
+#define gegl_chant_pointer(name, nick, blurb)
+#define gegl_chant_color(name, nick, def, blurb)
+#define gegl_chant_curve(name, nick, blurb)
+#define gegl_chant_seed(name, nick, blurb)
+#define gegl_chant_path(name, nick, blurb)
+#define gegl_chant_register_enum(enum_name) \
+typedef enum {
 
-struct _GeglChantO
-{
-  gpointer chant_data; /* Unused by the chanting framework can be used by operations
-                        * for storage of a private struct, (remember to clean up
-                        * in finalize). Also serves as a filler making sure that we
-                        * do not create an empty struct if there are no chanted properties.
-                        */
-#define gegl_chant_int(name, nick, min, max, def, blurb)                        gint               name;
-#define gegl_chant_int_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb)     gint               name;
-#define gegl_chant_double(name, nick, min, max, def, blurb)                     gdouble            name;
-#define gegl_chant_double_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb) gdouble   name;
-#define gegl_chant_boolean(name, nick, def, blurb)                              gboolean           name;
-#define gegl_chant_string(name, nick, def, blurb)                               gchar             *name;
-#define gegl_chant_enum(name, nick, enum, type, def, blurb)                     enum               name;
-#define gegl_chant_file_path(name, nick, def, blurb)                            gchar             *name;
-#define gegl_chant_multiline(name, nick, def, blurb)                            gchar             *name;
-#define gegl_chant_object(name,nick,  blurb)                                    GObject           *name;
-#define gegl_chant_pointer(name, nick, blurb)                                   gpointer           name;
-#define gegl_chant_color(name, nick, def, blurb)                                GeglColor         *name;
-#define gegl_chant_curve(name, nick, blurb)                                     GeglCurve         *name;
-#define gegl_chant_path(name, nick, blurb)                                      GeglPath          *name;\
-                                                                                guint path_changed_handler;
+#define enum_value(value, nick) \
+  value ,
+
+#define gegl_chant_register_enum_end(enum) \
+} enum ;
+
+#include GEGL_CHANT_C_FILE
+
+
+#undef gegl_chant_register_enum
+#undef enum_value
+#undef gegl_chant_register_enum_end
+
+#define gegl_chant_register_enum(enum_name)       \
+GType enum_name ## _get_type (void) G_GNUC_CONST; \
+GType enum_name ## _get_type (void)               \
+{                                                 \
+  static GType etype = 0;                         \
+  if (etype == 0) {                               \
+    static const GEnumValue values[] = {
+
+#define enum_value(value, nick) \
+      { value, nick, nick },
+
+#define gegl_chant_register_enum_end(enum)                \
+      { 0, NULL, NULL }                                   \
+    };                                                    \
+    etype = g_enum_register_static (#enum, values);       \
+  }                                                       \
+  return etype;                                           \
+}                                                         \
 
 #include GEGL_CHANT_C_FILE
 
@@ -373,6 +400,57 @@ struct _GeglChantO
 #undef gegl_chant_pointer
 #undef gegl_chant_color
 #undef gegl_chant_curve
+#undef gegl_chant_seed
+#undef gegl_chant_path
+#undef gegl_chant_register_enum
+#undef enum_value
+#undef gegl_chant_register_enum_end
+#define gegl_chant_register_enum(enum_name)
+#define enum_value(value, nick)
+#define gegl_chant_register_enum_end(enum)
+
+/* Properties */
+
+struct _GeglChantO
+{
+  gpointer chant_data; /* Unused by the chanting framework can be used by operations
+                        * for storage of a private struct, (remember to clean up
+                        * in finalize). Also serves as a filler making sure that we
+                        * do not create an empty struct if there are no chanted properties.
+                        */
+#define gegl_chant_int(name, nick, min, max, def, blurb)                        gint               name;
+#define gegl_chant_int_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb)     gint               name;
+#define gegl_chant_double(name, nick, min, max, def, blurb)                     gdouble            name;
+#define gegl_chant_double_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb) gdouble   name;
+#define gegl_chant_boolean(name, nick, def, blurb)                              gboolean           name;
+#define gegl_chant_string(name, nick, def, blurb)                               gchar             *name;
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)                    enum               name;
+#define gegl_chant_file_path(name, nick, def, blurb)                            gchar             *name;
+#define gegl_chant_multiline(name, nick, def, blurb)                            gchar             *name;
+#define gegl_chant_object(name,nick,  blurb)                                    GObject           *name;
+#define gegl_chant_pointer(name, nick, blurb)                                   gpointer           name;
+#define gegl_chant_color(name, nick, def, blurb)                                GeglColor         *name;
+#define gegl_chant_curve(name, nick, blurb)                                     GeglCurve         *name;
+#define gegl_chant_seed(name, nick, blurb)                                      gint               name;
+#define gegl_chant_path(name, nick, blurb)                                      GeglPath          *name;\
+                                                                                gulong path_changed_handler;
+
+#include GEGL_CHANT_C_FILE
+
+#undef gegl_chant_int
+#undef gegl_chant_int_ui
+#undef gegl_chant_double
+#undef gegl_chant_double_ui
+#undef gegl_chant_boolean
+#undef gegl_chant_string
+#undef gegl_chant_enum
+#undef gegl_chant_file_path
+#undef gegl_chant_multiline
+#undef gegl_chant_object
+#undef gegl_chant_pointer
+#undef gegl_chant_color
+#undef gegl_chant_curve
+#undef gegl_chant_seed
 #undef gegl_chant_path
 };
 
@@ -387,13 +465,14 @@ enum
 #define gegl_chant_double_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb) PROP_##name,
 #define gegl_chant_boolean(name, nick, def, blurb)                              PROP_##name,
 #define gegl_chant_string(name, nick, def, blurb)                               PROP_##name,
-#define gegl_chant_enum(name, nick, enum, type, def, blurb)                     PROP_##name,
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)                    PROP_##name,
 #define gegl_chant_file_path(name, nick, def, blurb)                            PROP_##name,
 #define gegl_chant_multiline(name, nick, def, blurb)                            PROP_##name,
 #define gegl_chant_object(name, nick, blurb)                                    PROP_##name,
 #define gegl_chant_pointer(name, nick, blurb)                                   PROP_##name,
 #define gegl_chant_color(name, nick, def, blurb)                                PROP_##name,
 #define gegl_chant_curve(name, nick, blurb)                                     PROP_##name,
+#define gegl_chant_seed(name, nick, blurb)                                      PROP_##name,
 #define gegl_chant_path(name, nick, blurb)                                      PROP_##name,
 
 #include GEGL_CHANT_C_FILE
@@ -411,6 +490,7 @@ enum
 #undef gegl_chant_pointer
 #undef gegl_chant_color
 #undef gegl_chant_curve
+#undef gegl_chant_seed
 #undef gegl_chant_path
   PROP_LAST
 };
@@ -451,7 +531,7 @@ get_property (GObject      *gobject,
     case PROP_##name:                                         \
       g_value_set_string (value, properties->name);           \
       break;
-#define gegl_chant_enum(name, nick, enum, type, def, blurb)   \
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)  \
     case PROP_##name:                                         \
       g_value_set_enum (value, properties->name);             \
       break;
@@ -479,11 +559,14 @@ get_property (GObject      *gobject,
     case PROP_##name:                                         \
       g_value_set_object (value, properties->name);           \
       break;
+#define gegl_chant_seed(name, nick, blurb)                    \
+    case PROP_##name:                                         \
+      g_value_set_int (value, properties->name);              \
+      break;
 #define gegl_chant_path(name, nick, blurb)                    \
     case PROP_##name:                                         \
-      if (!properties->name)properties->name = gegl_path_new (); /* this feels ugly */\
       g_value_set_object (value, properties->name);           \
-      break;/*XXX*/
+      break;
 
 #include GEGL_CHANT_C_FILE
 
@@ -500,6 +583,7 @@ get_property (GObject      *gobject,
 #undef gegl_chant_pointer
 #undef gegl_chant_color
 #undef gegl_chant_curve
+#undef gegl_chant_seed
 #undef gegl_chant_path
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
@@ -546,7 +630,7 @@ set_property (GObject      *gobject,
         g_free (properties->name);                                    \
       properties->name = g_strdup (g_value_get_string (value));       \
       break;
-#define gegl_chant_enum(name, nick, enum, type, def, blurb)           \
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)          \
     case PROP_##name:                                                 \
       properties->name = g_value_get_enum (value);                    \
       break;
@@ -584,21 +668,27 @@ set_property (GObject      *gobject,
          g_object_unref (properties->name);                           \
       properties->name = g_value_dup_object (value);                  \
       break;
+#define gegl_chant_seed(name, nick, blurb)                            \
+    case PROP_##name:                                                 \
+      properties->name = g_value_get_int (value);                     \
+    break;
 #define gegl_chant_path(name, nick, blurb)                            \
     case PROP_##name:                                                 \
       if (properties->name != NULL)                                   \
-        {\
-          if (properties->path_changed_handler) \
-            g_signal_handler_disconnect (G_OBJECT (properties->name), properties->path_changed_handler);\
-         properties->path_changed_handler = 0;\
-        }                                                             \
-      properties->name = NULL;                                        \
-      if (g_value_peek_pointer (value))                               \
         {                                                             \
-          properties->name = g_value_dup_object (value);              \
-          properties->path_changed_handler = g_signal_connect (G_OBJECT (properties->name), "changed",   \
-          G_CALLBACK(path_changed), gobject);     \
-         }\
+          if (properties->path_changed_handler)                       \
+            g_signal_handler_disconnect (G_OBJECT (properties->name), \
+                                         properties->path_changed_handler); \
+          properties->path_changed_handler = 0;                       \
+          g_object_unref (properties->name);                          \
+        }                                                             \
+      properties->name = g_value_dup_object (value);                  \
+      if (properties->name != NULL)                                   \
+        {                                                             \
+          properties->path_changed_handler =                          \
+            g_signal_connect (G_OBJECT (properties->name), "changed", \
+                              G_CALLBACK(path_changed), gobject);     \
+         }                                                            \
       break; /*XXX*/
 
 #include GEGL_CHANT_C_FILE
@@ -616,6 +706,7 @@ set_property (GObject      *gobject,
 #undef gegl_chant_pointer
 #undef gegl_chant_color
 #undef gegl_chant_curve
+#undef gegl_chant_seed
 #undef gegl_chant_path
 
     default:
@@ -641,7 +732,7 @@ static void gegl_chant_destroy_notify (gpointer data)
       g_free (properties->name);                    \
       properties->name = NULL;                      \
     }
-#define gegl_chant_enum(name, nick, enum, type, def, blurb)
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)
 #define gegl_chant_file_path(name, nick, def, blurb) \
   if (properties->name)                             \
     {                                               \
@@ -673,6 +764,7 @@ static void gegl_chant_destroy_notify (gpointer data)
       g_object_unref (properties->name);            \
       properties->name = NULL;                      \
     }
+#define gegl_chant_seed(name, nick, blurb)
 #define gegl_chant_path(name, nick, blurb)          \
   if (properties->name)                             \
     {                                               \
@@ -695,6 +787,7 @@ static void gegl_chant_destroy_notify (gpointer data)
 #undef gegl_chant_pointer
 #undef gegl_chant_color
 #undef gegl_chant_curve
+#undef gegl_chant_seed
 #undef gegl_chant_path
 
   g_slice_free (GeglChantO, properties);
@@ -720,7 +813,7 @@ gegl_chant_constructor (GType                  type,
 #define gegl_chant_double_ui(name, nick, min, max, def, ui_min, ui_max, ui_gamma, blurb)
 #define gegl_chant_boolean(name, nick, def, blurb)
 #define gegl_chant_string(name, nick, def, blurb)
-#define gegl_chant_enum(name, nick, enum, type, def, blurb)
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)
 #define gegl_chant_file_path(name, nick, def, blurb)
 #define gegl_chant_multiline(name, nick, def, blurb)
 #define gegl_chant_object(name, nick, blurb)
@@ -728,6 +821,7 @@ gegl_chant_constructor (GType                  type,
 #define gegl_chant_color(name, nick, def, blurb)              \
     if (properties->name == NULL) \
     {properties->name = gegl_color_new(def?def:"black");}
+#define gegl_chant_seed(name, nick, blurb)
 #define gegl_chant_path(name, nick, blurb)
 #define gegl_chant_curve(name, nick, blurb)
 
@@ -746,6 +840,7 @@ gegl_chant_constructor (GType                  type,
 #undef gegl_chant_pointer
 #undef gegl_chant_color
 #undef gegl_chant_curve
+#undef gegl_chant_seed
 #undef gegl_chant_path
 
   g_object_set_data_full (obj, "chant-data", obj, gegl_chant_destroy_notify);
@@ -810,10 +905,10 @@ gegl_chant_class_intern_init (gpointer klass)
                                                         G_PARAM_READWRITE |  \
                                                         G_PARAM_CONSTRUCT |  \
                                                         GEGL_PARAM_PAD_INPUT)));
-#define gegl_chant_enum(name, nick, enum, type, def, blurb)                  \
+#define gegl_chant_enum(name, nick, enum, enum_name, def, blurb)                 \
   g_object_class_install_property (object_class, PROP_##name,                \
                                    g_param_spec_enum (#name, nick, blurb,    \
-                                                      type,                  \
+                                                      enum_name ## _get_type (), \
                                                       def,                   \
                                                       (GParamFlags) (        \
                                                       G_PARAM_READWRITE |    \
@@ -859,6 +954,13 @@ gegl_chant_class_intern_init (gpointer klass)
                                                           G_PARAM_READWRITE |\
                                                           G_PARAM_CONSTRUCT | \
                                                           GEGL_PARAM_PAD_INPUT)));
+#define gegl_chant_seed(name, nick, blurb)                                   \
+  g_object_class_install_property (object_class, PROP_##name,                \
+                                   gegl_param_spec_seed (#name, nick, blurb, \
+                                                     (GParamFlags) (         \
+                                                     G_PARAM_READWRITE |     \
+                                                     G_PARAM_CONSTRUCT |     \
+                                                     GEGL_PARAM_PAD_INPUT)));
 #define gegl_chant_path(name, nick, blurb)                                 \
   g_object_class_install_property (object_class, PROP_##name,                \
                                    gegl_param_spec_path (#name, nick, blurb,\
@@ -868,13 +970,15 @@ gegl_chant_class_intern_init (gpointer klass)
                                                           G_PARAM_CONSTRUCT |\
                                                           GEGL_PARAM_PAD_INPUT)));
 #define gegl_chant_curve(name, nick, blurb)                                  \
+  { GeglCurve *_gegl_chant_default_curve = gegl_curve_new_default ();        \
   g_object_class_install_property (object_class, PROP_##name,                \
                                    gegl_param_spec_curve (#name, nick, blurb,\
-                                                          gegl_curve_default_curve(),\
+                                                          _gegl_chant_default_curve,\
                                                           (GParamFlags) (    \
                                                           G_PARAM_READWRITE |\
                                                           G_PARAM_CONSTRUCT |\
-                                                          GEGL_PARAM_PAD_INPUT)));
+                                                          GEGL_PARAM_PAD_INPUT)));\
+  g_object_unref (_gegl_chant_default_curve); }
 
 #include GEGL_CHANT_C_FILE
 
@@ -891,6 +995,7 @@ gegl_chant_class_intern_init (gpointer klass)
 #undef gegl_chant_pointer
 #undef gegl_chant_color
 #undef gegl_chant_curve
+#undef gegl_chant_seed
 #undef gegl_chant_path
 }
 
