@@ -14,7 +14,9 @@ typedef enum {
   GEGL_DEBUG_PROCESSOR       = 1 << 4,
   GEGL_DEBUG_CACHE           = 1 << 5,
   GEGL_DEBUG_MISC            = 1 << 6,
-  GEGL_DEBUG_INVALIDATION    = 1 << 7
+  GEGL_DEBUG_INVALIDATION    = 1 << 7,
+  GEGL_DEBUG_OPENCL          = 1 << 8,
+  GEGL_DEBUG_BUFFER_ALLOC    = 1 << 9
 } GeglDebugFlag;
 
 /* only compiled in from gegl-init.c but kept here to
@@ -30,14 +32,18 @@ static const GDebugKey gegl_debug_keys[] = {
   { "tile-backend",  GEGL_DEBUG_TILE_BACKEND},
   { "processor",     GEGL_DEBUG_PROCESSOR},
   { "invalidation",  GEGL_DEBUG_INVALIDATION},
+  { "opencl",        GEGL_DEBUG_OPENCL},
+  { "buffer-alloc",  GEGL_DEBUG_BUFFER_ALLOC},
   { "all",           GEGL_DEBUG_PROCESS|
                      GEGL_DEBUG_BUFFER_LOAD|
                      GEGL_DEBUG_BUFFER_SAVE|
                      GEGL_DEBUG_TILE_BACKEND|
                      GEGL_DEBUG_PROCESSOR|
-                     GEGL_DEBUG_CACHE},
+                     GEGL_DEBUG_CACHE|
+                     GEGL_DEBUG_OPENCL|
+                     GEGL_DEBUG_BUFFER_ALLOC},
 };
-#endif /* GEGL_ENABLE_DEBUG */
+#endif /* __GEGL_INIT_C */
 
 #if defined(__cplusplus) && defined(GEGL_ISO_CXX_VARIADIC_MACROS)
 #  define GEGL_ISO_VARIADIC_MACROS 1
@@ -107,6 +113,7 @@ gegl_lookup_debug_string  (guint type)
               key = gegl_debug_keys[i].key;
               break;
             }
+          i++;
         }
     }
 
