@@ -75,7 +75,7 @@ struct _GeglNode
   /* Whether result is cached or not, inherited by children */
   gboolean        dont_cache;
 
-  GMutex         *mutex;
+  GMutex          mutex;
 
   /*< private >*/
   GeglNodePrivate *priv;
