@@ -25,6 +25,7 @@
 
 
 #include "gegl-buffer.h"
+#include "opencl/gegl-cl.h"
 
 G_BEGIN_DECLS
 
@@ -139,7 +140,10 @@ struct _GeglOperationClass
   GeglNode*     (*detect)                    (GeglOperation       *operation,
                                               gint                 x,
                                               gint                 y);
-  gpointer      pad[10];
+
+  GeglClRunData *cl_data;
+
+  gpointer      pad[9];
 };
 
 
@@ -207,8 +211,14 @@ const Babl *    gegl_operation_get_format    (GeglOperation *operation,
 const gchar *   gegl_operation_get_name      (GeglOperation *operation);
 
 
+/* checks the incoming Babl format on a given pad, can be used in the prepare
+ * stage to make format dependent decisions
+ */
+const Babl  * gegl_operation_get_source_format (GeglOperation *operation,
+                                                const gchar   *padname);
+
 /* retrieves the node providing data to a named input pad */
-GeglNode      * gegl_operation_get_source_node (GeglOperation *operation,
+GeglNode    * gegl_operation_get_source_node   (GeglOperation *operation,
                                                 const gchar   *pad_name);
 
 GParamSpec ** gegl_operation_list_properties   (const gchar *operation_type,
@@ -246,13 +256,18 @@ void     gegl_operation_invalidate       (GeglOperation       *operation,
                                           const GeglRectangle *roi,
                                           gboolean             clear_cache);
 
+gboolean gegl_operation_cl_set_kernel_args (GeglOperation *operation,
+                                            cl_kernel      kernel,
+                                            gint          *p,
+                                            cl_int        *err);
+
 /* internal utility functions used by gegl, these should not be used
  * externally */
 gboolean gegl_operation_calc_need_rects      (GeglOperation       *operation,
                                               gpointer             context_id);
-void     gegl_operation_path_prop_changed    (GeglPath            *path,
-                                              GeglOperation       *operation);
-
+gboolean gegl_can_do_inplace_processing      (GeglOperation       *operation,
+                                              GeglBuffer          *input,
+                                              const GeglRectangle *result);
 G_END_DECLS
 
 /***
