@@ -33,7 +33,7 @@ gegl_chant_double (contrast, _("Contrast"),
                   _("The amount of contrast compression"))
 gegl_chant_double (saturation, _("Saturation"),
                   0.0, 2.0, 0.8,
-                  _("Global colour saturation factor"))
+                  _("Global color saturation factor"))
 gegl_chant_double (detail, _("Detail"),
                   1.0, 99.0, 1.0,
                   _("Level of emphasis on image gradient details"))
@@ -1576,9 +1576,10 @@ mantiuk06_get_cached_region (GeglOperation        *operation,
 
 static gboolean
 mantiuk06_process (GeglOperation       *operation,
-                    GeglBuffer          *input,
-                    GeglBuffer          *output,
-                    const GeglRectangle *result)
+                   GeglBuffer          *input,
+                   GeglBuffer          *output,
+                   const GeglRectangle *result,
+                   gint                 level)
 {
   const GeglChantO *o          = GEGL_CHANT_PROPERTIES (operation);
   const gint        pix_stride = 4; /* RGBA */
@@ -1594,18 +1595,18 @@ mantiuk06_process (GeglOperation       *operation,
 
   /* Obtain the pixel data */
   lum = g_new (gfloat, result->width * result->height),
-  gegl_buffer_get (input, 1.0, result, babl_format ("Y float"),
-                   lum, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (input, result, 1.0, babl_format ("Y float"),
+                   lum, GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   pix = g_new (gfloat, result->width * result->height * pix_stride);
-  gegl_buffer_get (input, 1.0, result, babl_format (OUTPUT_FORMAT),
-                   pix, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (input, result, 1.0, babl_format (OUTPUT_FORMAT),
+                   pix, GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   mantiuk06_contmap (result->width, result->height, pix, lum,
                      o->contrast, o->saturation, FALSE, 200, 1e-3, NULL);
 
   /* Cleanup and set the output */
-  gegl_buffer_set (output, result, babl_format (OUTPUT_FORMAT), pix,
+  gegl_buffer_set (output, result, 0, babl_format (OUTPUT_FORMAT), pix,
                    GEGL_AUTO_ROWSTRIDE);
   g_free (pix);
   g_free (lum);
@@ -1631,13 +1632,15 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_required_for_output = mantiuk06_get_required_for_output;
   operation_class->get_cached_region       = mantiuk06_get_cached_region;
 
-  operation_class->name        = "gegl:mantiuk06";
-  operation_class->categories  = "tonemapping";
-  operation_class->description =
+  gegl_operation_class_set_keys (operation_class,
+      "name"       , "gegl:mantiuk06",
+      "categories" , "tonemapping",
+      "description",
         _("Adapt an image, which may have a high dynamic range, for "
           "presentation using a low dynamic range. This operator constrains "
           "contrasts across multiple spatial frequencies, producing "
-          "luminance within the range 0.0-1.0");
+          "luminance within the range 0.0-1.0"),
+        NULL);
 }
 
 #endif
