@@ -23,8 +23,8 @@
 gegl_chant_double (scaling, _("Scaling"), 0.0, 5000.0, 1.0,
        _("scaling factor of displacement, indicates how large spatial"
          " displacement a relative mapping value of 1.0 corresponds to."))
-gegl_chant_enum (sampler_type, _("Sampler"), GeglSamplerType, GEGL_TYPE_SAMPLER_TYPE,
-                 GEGL_SAMPLER_CUBIC, _("Sampler used internaly"))
+gegl_chant_enum (sampler_type, _("Sampler"), GeglSamplerType, gegl_sampler_type,
+                 GEGL_SAMPLER_CUBIC, _("Sampler used internally"))
 
 #else
 
@@ -39,7 +39,7 @@ gegl_chant_enum (sampler_type, _("Sampler"), GeglSamplerType, GEGL_TYPE_SAMPLER_
 static void
 prepare (GeglOperation *operation)
 {
-  Babl *format = babl_format ("RGBA float");
+  const Babl *format = babl_format ("RGBA float");
 
   gegl_operation_set_format (operation, "input", format);
   gegl_operation_set_format (operation, "aux", babl_format_n (babl_type ("float"), 2));
@@ -61,10 +61,11 @@ process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *aux,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
   GeglChantO           *o = GEGL_CHANT_PROPERTIES (operation);
-  Babl                 *format_io, *format_coords;
+  const Babl           *format_io, *format_coords;
   GeglSampler          *sampler;
   GeglBufferIterator   *it;
   gint                  index_in, index_out, index_coords;
@@ -76,11 +77,11 @@ process (GeglOperation       *operation,
 
   if (aux != NULL)
     {
-      it = gegl_buffer_iterator_new (output, result, format_io, GEGL_BUFFER_WRITE);
+      it = gegl_buffer_iterator_new (output, result, level, format_io, GEGL_BUFFER_WRITE, GEGL_ABYSS_NONE);
       index_out = 0;
 
-      index_coords = gegl_buffer_iterator_add (it, aux, result, format_coords, GEGL_BUFFER_READ);
-      index_in = gegl_buffer_iterator_add (it, input, result, format_io, GEGL_BUFFER_READ);
+      index_coords = gegl_buffer_iterator_add (it, aux, result, level, format_coords, GEGL_BUFFER_READ, GEGL_ABYSS_NONE);
+      index_in = gegl_buffer_iterator_add (it, input, result, level, format_io, GEGL_BUFFER_READ, GEGL_ABYSS_NONE);
 
       while (gegl_buffer_iterator_next (it))
         {
@@ -108,7 +109,8 @@ process (GeglOperation       *operation,
                 {
                   gegl_sampler_get (sampler, x+coords[0] * scaling,
                                              y+coords[1] * scaling,
-                                             NULL, out);
+                                             NULL, out,
+                                             GEGL_ABYSS_NONE);
                 }
 
               coords += 2;
@@ -141,6 +143,22 @@ gegl_chant_class_init (GeglChantClass *klass)
 {
   GeglOperationClass         *operation_class;
   GeglOperationComposerClass *composer_class;
+  gchar                      *composition = "<?xml version='1.0' encoding='UTF-8'?>"
+    "<gegl>"
+    "<node operation='gegl:map-relative'>"
+    "  <params>"
+    "    <param name='scaling'>100</param>"
+    "  </params>"
+    "  <node operation='gegl:load'>"
+    "    <params><param name='path'>standard-aux.png</param></params>"
+    "  </node>"
+    "</node>"
+    "<node operation='gegl:load'>"
+    "  <params>"
+    "    <param name='path'>standard-input.png</param>"
+    "  </params>"
+    "</node>"
+    "</gegl>";
 
   operation_class = GEGL_OPERATION_CLASS (klass);
   composer_class  = GEGL_OPERATION_COMPOSER_CLASS (klass);
@@ -149,9 +167,11 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->prepare = prepare;
   operation_class->get_required_for_output = get_required_for_output;
 
-  operation_class->name        = "gegl:map-relative";
-
-  operation_class->categories  = "transform";
-  operation_class->description = _("sample input with an auxiliary buffer that contain relative source coordinates");
+  gegl_operation_class_set_keys (operation_class,
+    "name"       , "gegl:map-relative",
+    "categories" , "transform",
+    "description", _("sample input with an auxiliary buffer that contain relative source coordinates"),
+    "reference-composition", composition,
+    NULL);
 }
 #endif
