@@ -15,10 +15,15 @@
  *
  * Ali Alsam, Hans Jakob Rivertz, Øyvind Kolås (c) 2011
  */
+#include "config.h"
+#include <glib/gi18n-lib.h>
+
+#include "config.h"
+#include <glib/gi18n-lib.h>
 
 #ifdef GEGL_CHANT_PROPERTIES
 
-gegl_chant_int (iterations, "Strength", 1, 32, 4, "How many iteratarion to run the algorithm.")
+gegl_chant_int_ui (iterations, _("Strength"), 0, 32, 4, 0, 8, 1, _("How many iteratarions to run the algorithm with"))
 
 #else
 
@@ -144,15 +149,158 @@ static void prepare (GeglOperation *operation)
   gegl_operation_set_format (operation, "output", babl_format ("R'G'B'A float"));
 }
 
+#include "opencl/gegl-cl.h"
+#include "buffer/gegl-buffer-cl-iterator.h"
+
+#include "opencl/noise-reduction.cl.h"
+
+static GeglClRunData *cl_data = NULL;
+
+static gboolean
+cl_noise_reduction (cl_mem                in_tex,
+                    cl_mem                aux_tex,
+                    cl_mem                out_tex,
+                    size_t                global_worksize,
+                    const GeglRectangle  *src_roi,
+                    const GeglRectangle  *roi,
+                    const int             iterations)
+{
+  int i = 0;
+  size_t gbl_size_tmp[2];
+
+  cl_int n_src_stride  = roi->width + iterations * 2;
+  cl_int cl_err = 0;
+
+  cl_mem temp_tex, tmptex;
+
+  gint stride = 16; /*R'G'B'A float*/
+
+  if (!cl_data)
+    {
+      const char *kernel_name[] ={"noise_reduction_cl","transfer", NULL};
+      cl_data = gegl_cl_compile_and_build(noise_reduction_cl_source, kernel_name);
+    }
+  if (!cl_data)  return TRUE;
+
+  temp_tex = gegl_clCreateBuffer (gegl_cl_get_context(),
+                                  CL_MEM_READ_WRITE,
+                                  src_roi->width * src_roi->height * stride,
+                                  NULL, &cl_err);
+  CL_CHECK;
+
+  cl_err = gegl_clEnqueueCopyBuffer(gegl_cl_get_command_queue(),
+                                    in_tex , temp_tex , 0 , 0 ,
+                                    src_roi->width * src_roi->height * stride,
+                                    0, NULL, NULL);
+  CL_CHECK;
+
+  tmptex = temp_tex;
+  for (i = 0;i<iterations;i++)
+    {
+      if (i > 0)
+      {
+        cl_mem temp = aux_tex;
+        aux_tex     = temp_tex;
+        temp_tex    = temp;
+      }
+      gbl_size_tmp[0] = roi->width  + 2 * (iterations - 1 -i);
+      gbl_size_tmp[1] = roi->height + 2 * (iterations - 1 -i);
+
+      cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 0, sizeof(cl_mem), (void*)&temp_tex);
+      CL_CHECK;
+      cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 1, sizeof(cl_int), (void*)&n_src_stride);
+      CL_CHECK;
+      cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 2, sizeof(cl_mem), (void*)&aux_tex);
+      CL_CHECK;
+      cl_err = gegl_clSetKernelArg(cl_data->kernel[0], 3, sizeof(cl_int), (void*)&n_src_stride);
+      CL_CHECK;
+
+      cl_err = gegl_clEnqueueNDRangeKernel(gegl_cl_get_command_queue(), cl_data->kernel[0],
+                                           2, NULL, gbl_size_tmp, NULL,
+                                           0, NULL, NULL);
+      CL_CHECK;
+    }
+
+  gbl_size_tmp[0] = roi->width ;
+  gbl_size_tmp[1] = roi->height;
+
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[1], 0, sizeof(cl_mem), (void*)&aux_tex);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[1], 1, sizeof(cl_int), (void*)&n_src_stride);
+  CL_CHECK;
+  cl_err = gegl_clSetKernelArg(cl_data->kernel[1], 2, sizeof(cl_mem), (void*)&out_tex);
+  CL_CHECK;
+
+  cl_err = gegl_clEnqueueNDRangeKernel(gegl_cl_get_command_queue(), cl_data->kernel[1],
+                                       2, NULL, gbl_size_tmp, NULL,
+                                       0, NULL, NULL);
+  CL_CHECK;
+
+  cl_err = gegl_clFinish(gegl_cl_get_command_queue());
+  CL_CHECK;
+
+  if (tmptex) cl_err = gegl_clReleaseMemObject (tmptex);
+  CL_CHECK;
+
+  return FALSE;
+
+error:
+  return TRUE;
+}
+
+static gboolean
+cl_process (GeglOperation       *operation,
+      GeglBuffer                *input,
+      GeglBuffer                *output,
+      const GeglRectangle       *result)
+{
+  const Babl *in_format  = gegl_operation_get_format (operation, "input");
+  const Babl *out_format = gegl_operation_get_format (operation, "output");
+  gint err;
+  gint j;
+
+  GeglOperationAreaFilter *op_area = GEGL_OPERATION_AREA_FILTER (operation);
+  GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
+
+  GeglBufferClIterator *i = gegl_buffer_cl_iterator_new (output,   result, out_format, GEGL_CL_BUFFER_WRITE);
+  gint read = gegl_buffer_cl_iterator_add_2 (i, input, result, in_format,  GEGL_CL_BUFFER_READ,
+                                             op_area->left, op_area->right, op_area->top, op_area->bottom, GEGL_ABYSS_NONE);
+  gint aux  = gegl_buffer_cl_iterator_add_2 (i, NULL, result, in_format,  GEGL_CL_BUFFER_AUX,
+                                             op_area->left, op_area->right, op_area->top, op_area->bottom, GEGL_ABYSS_NONE);
+
+  while (gegl_buffer_cl_iterator_next (i, &err))
+  {
+    if (err) return FALSE;
+    for (j=0; j < i->n; j++)
+      {
+        err = cl_noise_reduction(i->tex[read][j],
+                                 i->tex[aux][j],
+                                 i->tex[0][j],
+                                 i->size[0][j],
+                                 &i->roi[read][j],
+                                 &i->roi[0][j],
+                                 o->iterations);
+        if (err)
+        {
+          g_warning("[OpenCL] Error in gegl:noise-reduction");
+          return FALSE;
+        }
+      }
+  }
+  return TRUE;
+}
+
 #define INPLACE 1
 
 static gboolean
 process (GeglOperation       *operation,
          GeglBuffer          *input,
          GeglBuffer          *output,
-         const GeglRectangle *result)
+         const GeglRectangle *result,
+         gint                 level)
 {
   GeglChantO   *o = GEGL_CHANT_PROPERTIES (operation);
+
   int iteration;
   int stride;
   float *src_buf;
@@ -160,6 +308,11 @@ process (GeglOperation       *operation,
   float *dst_buf;
 #endif
   GeglRectangle rect;
+
+  if (gegl_cl_is_accelerated ())
+    if(cl_process(operation, input, output, result))
+      return TRUE;
+
   rect = *result;
 
   stride = result->width + o->iterations * 2;
@@ -176,8 +329,8 @@ process (GeglOperation       *operation,
     rect.y      -= o->iterations;
     rect.width  += o->iterations*2;
     rect.height += o->iterations*2;
-    gegl_buffer_get (input, 1.0, &rect, babl_format ("R'G'B'A float"),
-                     src_buf, stride * 4 * 4);
+    gegl_buffer_get (input, &rect, 1.0, babl_format ("R'G'B'A float"),
+                     src_buf, stride * 4 * 4, GEGL_ABYSS_NONE);
   }
 
   for (iteration = 0; iteration < o->iterations; iteration++)
@@ -200,7 +353,7 @@ process (GeglOperation       *operation,
 #endif
     }
 
-  gegl_buffer_set (output , result, babl_format ("R'G'B'A float"),
+  gegl_buffer_set (output, result, 0, babl_format ("R'G'B'A float"),
 #ifndef INPLACE
                    src_buf,
 #else
@@ -239,11 +392,15 @@ gegl_chant_class_init (GeglChantClass *klass)
 
   filter_class->process   = process;
   operation_class->prepare = prepare;
+  operation_class->opencl_support = TRUE;
+
   operation_class->get_bounding_box = get_bounding_box;
 
-  operation_class->name        = "gegl:noise-reduction";
-  operation_class->categories  = "enhance";
-  operation_class->description = "Anisotropic like smoothing operation";
+  gegl_operation_class_set_keys (operation_class,
+    "name"       , "gegl:noise-reduction",
+    "categories" , "enhance",
+    "description", "Anisotropic like smoothing operation",
+    NULL);
 }
 
 #endif
