@@ -57,6 +57,7 @@ gegl_chant_int    (active_levels, _("Active Levels"),
  */
 #if defined(HAVE_UMFPACK_H)
 #include <umfpack.h>
+#define SuiteSparse_long UF_long
 #elif defined (HAVE_SUITESPARSE_UMFPACK_H)
 #include <suitesparse/umfpack.h>
 #endif
@@ -105,8 +106,10 @@ static const gchar *FORMAT_AUX    = "Y'A double";
 static const gchar *FORMAT_INPUT  = "R'G'B' double";
 static const gchar *FORMAT_OUTPUT = "Y' double";
 
-static const guint AUX_VALUE = 0;
-static const guint AUX_ALPHA = 1;
+static const guint   AUX_VALUE = 0;
+static const guint   AUX_ALPHA = 1;
+static const gdouble AUX_BACKGROUND = 0.1;
+static const gdouble AUX_FOREGROUND = 0.9;
 
 
 /* Threshold below which we consider the Y channel to be undefined (or
@@ -848,8 +851,8 @@ matting_sparse_new (guint cols, guint rows, guint elems)
   sparse_t *s = g_new (sparse_t, 1);
   s->columns  = cols;
   s->rows     = rows;
-  s->col_idx  = g_new  (UF_long, cols + 1);
-  s->row_idx  = g_new  (UF_long, elems);
+  s->col_idx  = g_new  (SuiteSparse_long, cols + 1);
+  s->row_idx  = g_new  (SuiteSparse_long, elems);
   s->values   = g_new0 (gdouble, elems);
 
   return s;
@@ -943,18 +946,18 @@ matting_get_laplacian (const gdouble       *restrict image,
                        const gdouble        epsilon,
                        const gdouble        lambda)
 {
-  gint      diameter     = radius * 2 + 1,
-            window_elems = diameter * diameter,
-            image_elems  = roi->width * roi->height,
-            i, j, k, x, y,
-            status;
-  UF_long  *trip_col,
-           *trip_row;
-  glong     trip_nz = 0,
-            trip_cursor = 0,
-            trip_masked = 0;
-  gdouble  *trip_val;
-  sparse_t *laplacian;
+  gint              diameter     = radius * 2 + 1,
+                    window_elems = diameter * diameter,
+                    image_elems  = roi->width * roi->height,
+                    i, j, k, x, y,
+                    status;
+  SuiteSparse_long *trip_col,
+                   *trip_row;
+  glong             trip_nz = 0,
+                    trip_cursor = 0,
+                    trip_masked = 0;
+  gdouble          *trip_val;
+  sparse_t         *laplacian;
 
   gdouble       mean[COMPONENTS_INPUT],
          mean_matrix[COMPONENTS_INPUT][COMPONENTS_INPUT],
@@ -979,8 +982,8 @@ matting_get_laplacian (const gdouble       *restrict image,
   trip_nz   = trip_masked * window_elems * window_elems;
   trip_nz  += image_elems; // Sparse diagonal and row summing at conclusion
 
-  trip_col  = g_new  (UF_long, trip_nz);
-  trip_row  = g_new  (UF_long, trip_nz);
+  trip_col  = g_new  (SuiteSparse_long, trip_nz);
+  trip_row  = g_new  (SuiteSparse_long, trip_nz);
   trip_val  = g_new0 (gdouble, trip_nz);
 
   /* Compute the contribution of each pixel in the image to the laplacian */
@@ -1066,10 +1069,10 @@ matting_get_laplacian (const gdouble       *restrict image,
           for (y = 0; y < window_elems; ++y)
             for (x = 0; x < window_elems; ++x)
               {
-                UF_long yx = y % diameter,
-                        yy = y / diameter,
-                        xx = x % diameter,
-                        xy = x / diameter;
+                SuiteSparse_long yx = y % diameter,
+                                 yy = y / diameter,
+                                 xx = x % diameter,
+                                 xy = x / diameter;
 
                 g_return_val_if_fail (trip_cursor < trip_nz, FALSE);
                 trip_col[trip_cursor] = (i - radius + yx) + (j - radius + yy) * roi->width,
@@ -1267,6 +1270,15 @@ matting_solve_level (gdouble             *restrict pixels,
   gint     i;
   gdouble *new_alpha    = NULL,
           *eroded_alpha = NULL;
+  gdouble value;
+
+  /* add alpha to trimap because algo is need it */
+  for (i = 0; i < region->width * region->height; ++i)
+  {
+    value = trimap[i * COMPONENTS_AUX + AUX_VALUE];
+    if (value > AUX_BACKGROUND && value < AUX_FOREGROUND)
+        trimap[i * COMPONENTS_AUX + AUX_ALPHA] = 0.0;
+  }
 
   if (region->width  < MIN_LEVEL_DIAMETER ||
       region->height < MIN_LEVEL_DIAMETER)
@@ -1364,7 +1376,8 @@ matting_process (GeglOperation       *operation,
                  GeglBuffer          *input_buf,
                  GeglBuffer          *aux_buf,
                  GeglBuffer          *output_buf,
-                 const GeglRectangle *result)
+                 const GeglRectangle *result,
+                 gint                 level)
 {
   const GeglChantO *o = GEGL_CHANT_PROPERTIES (operation);
   gdouble          *input   = NULL,
@@ -1385,14 +1398,14 @@ matting_process (GeglOperation       *operation,
   input  = g_new (gdouble, result->width * result->height * COMPONENTS_INPUT);
   trimap = g_new (gdouble, result->width * result->height * COMPONENTS_AUX);
 
-  gegl_buffer_get (input_buf, 1.0, result, babl_format (FORMAT_INPUT), input, GEGL_AUTO_ROWSTRIDE);
-  gegl_buffer_get (  aux_buf, 1.0, result, babl_format (FORMAT_AUX),  trimap, GEGL_AUTO_ROWSTRIDE);
+  gegl_buffer_get (input_buf, result, 1.0, babl_format (FORMAT_INPUT), input, GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
+  gegl_buffer_get (  aux_buf, result, 1.0, babl_format (FORMAT_AUX),  trimap, GEGL_AUTO_ROWSTRIDE, GEGL_ABYSS_NONE);
 
   output = matting_solve_level (input, trimap, result,
                                 MIN (o->active_levels, o->levels), o->levels,
                                 o->radius, powf (10, o->epsilon), o->lambda,
                                 o->threshold);
-  gegl_buffer_set (output_buf, result, babl_format (FORMAT_OUTPUT), output,
+  gegl_buffer_set (output_buf, result, 0, babl_format (FORMAT_OUTPUT), output,
                    GEGL_AUTO_ROWSTRIDE);
 
   success = TRUE;
@@ -1420,12 +1433,14 @@ gegl_chant_class_init (GeglChantClass *klass)
   operation_class->get_required_for_output = matting_get_required_for_output;
   operation_class->get_cached_region       = matting_get_cached_region;
 
-  operation_class->name        = "gegl:matting-levin";
-  operation_class->categories  = "misc";
-  operation_class->description =
+  gegl_operation_class_set_keys (operation_class,
+  "name"        , "gegl:matting-levin",
+  "categories"  , "misc",
+  "description" ,
         _("Given a sparse user supplied tri-map and an input image, create a "
           "foreground alpha mat. Set white as selected, black as unselected, "
-          "for the tri-map.");
+          "for the tri-map."),
+        NULL);
 }
 
 
